/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2012                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>
#include<vcg/complex/algorithms/update/topology.h>
#include<vcg/complex/algorithms/update/normal.h>
using namespace vcg;

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public UsedTypes<Use<MyVertex>   ::AsVertexType,
                                           Use<MyEdge>     ::AsEdgeType,
                                           Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public Vertex< MyUsedTypes, vertex::Coord3f, vertex::Normal3f,
                                              vertex::Color4b, vertex::VFAdj, vertex::BitFlags  >{};
class MyFace    : public Face<   MyUsedTypes, face::FFAdj, face::VFAdj,
    face::Normal3f, face::VertexRef, face::BitFlags > {};
class MyEdge    : public Edge<   MyUsedTypes> {};

class MyMesh    : public tri::TriMesh< std::vector<MyVertex>,
                                       std::vector<MyFace> ,
                                       std::vector<MyEdge>  > {};


int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.obj>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;

  if(tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=
     tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  tri::RequirePerVertexNormal(m);
  tri::UpdateNormal<MyMesh>::PerVertexNormalized(m);
  tri::UpdateTopology<MyMesh>::FaceFace(m);
  tri::UpdateTopology<MyMesh>::VertexFace(m);

  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  printf( "Mesh has %i vert and %i faces\n", m.VN(), m.FN() );

  // Task:
  // Find all the corner vertices
  // A vertex is a corner if there are more than two incident edges
  // where the normals of the two adjacent faces differ more than a
  // given threshold angle
  int edgeCnt=0;
  const float thrRad = math::ToRad(60.0f);
  for(MyMesh::VertexIterator vi=m.vert.begin();vi!=m.vert.end();++vi)
  {
   MyFace *fp = vi->VFp();
   face::Pos<MyFace> startPos(fp,&*vi);
   face::Pos<MyFace> iterPos = startPos;
   int creaseCnt=0;
   do
   {
     iterPos.FlipE();
     Point3f N0 = iterPos.F()->N();
     iterPos.FlipF();
     Point3f N1 = iterPos.F()->N();
     float angleRad = Angle(N0,N1);
     if(angleRad > thrRad)
      creaseCnt++;
     edgeCnt++;
   }
   while(iterPos!=startPos);
   switch(creaseCnt)
   {
   case 0: vi->C() = Color4b::White; break;
   case 1: vi->C() = Color4b::Gray; break;
   case 2: vi->C() = Color4b::Yellow; break;
   case 3: vi->C() = Color4b::Red; break;
   default: vi->C() = Color4b::Black; break;
   }
  }
  printf("Edges %i\n",edgeCnt/2);
  printf("V-E+F = %i - %i + %i = %i\n",
         m.VN(), edgeCnt/2, m.FN(), m.VN()-(edgeCnt/2) + m.FN());

  tri::io::ExporterOFF<MyMesh>::Save(m,"out.off",tri::io::Mask::IOM_VERTCOLOR);
  return 0;
}
